//+------------------------------------------------------------------+
//|                                                   TakeProfit.mq4 |
//|                               Copyright  2006, Taylor Stockwell |
//|                                               stockwet@yahoo.com |
//|                                                    Version: 2.4  |
//+------------------------------------------------------------------+
#property copyright "Copyright  2006, Taylor Stockwell"
#property link      "stockwet@yahoo.com"


/*
Updates:

   11/2/2006:: Fixed bug that caused the sl var to be set to 1 when a 
               pending order was > than the First_Stop_Target value.
   1/23/2007:: Fixed pip calculation. The old way of calculating pips 
               could be off by 1 pip, resulting in missed TP or move 
               stops. The new calculation is more accurate.
   5/31/2007:: Lousan:  Added a Third Stop Loss.  Renamed EA to MTP 
               to differentiate it from Stockwell's version.
   6/01/2007:: Lousan:  Added a pip by pip trailing stop loss to kick 
               in after the last fixed Stop Loss is in place.
   7/27/2007:: Lousan:  Ver 1.2  Added Version # and Revision date and 
               Pre-Set Name to display.   
   8/09/2007:: Lousan:  Ver 1.3  Trailing stop was not being moved when
               fixed stops were zero.
   8/14/2007:: Lousan: Cosmetic change to Legend
                                   
*/

//=============== VARS external
extern string PreSet_Name = "Default";
extern int First_TP_Target = 100;
extern int TP_Target_Increment = 1000;
extern double Close_Lots = 1.0;
extern bool Move_Stops = true;
extern int First_Stop_Target = 100;
extern int First_Stop = 10;
extern int Second_Stop_Target = 250;
extern int Second_Stop = 125;
extern int Third_Stop_Target = 400;
extern int Third_Stop = 250;
extern int Trail_Stop_By = 125;
extern bool Use_Max_Loss = true;
extern int Max_Loss = 200;
extern int Magic_Number=0;

//=============== VARS internal
int nextTP;
bool sl;
int range = 5;
int multiplier;
int Trail_Stop;
int Trail_Trigger;
string trailon;
string version="MTP 1.3  August 14, 2007   ";

// OrderType == 1 is OP_SELL




//=============== FUNCTION init
int init()
  {
   if(First_Stop==0 && First_Stop_Target !=0) First_Stop=1;
   if(Max_Loss < 0) Max_Loss=(-1*Max_Loss);
   sl=0;
   nextTP = First_TP_Target;
   Trail_Stop = 0;
   trailon=" ";
   getMaxLoss();
   if(Third_Stop!=0) Trail_Trigger=Third_Stop;
   else if(Second_Stop!=0) Trail_Trigger=Second_Stop;
   else if(First_Stop!=0) Trail_Trigger=First_Stop;
   else Trail_Trigger=0;
   Comment(version, 
   "\nPreSet ..... ", PreSet_Name,
   "\nClose Lots:  ", Close_Lots,
   "\nTP:   ", First_TP_Target, "  ", TP_Target_Increment,
   "\nSL1:  ", First_Stop_Target, "  ", First_Stop,
   "\nSL2:  ", Second_Stop_Target, "  ", Second_Stop,
   "\nSL3:  ", Third_Stop_Target,  "  ", Third_Stop,
   "\nTSL:  ", Trail_Stop_By,
   "\nMax Loss:  ", Max_Loss);
  }
//== end function


//=============== FUNCTION deinit
int deinit()
  {
//----
   sl=0;
   nextTP = First_TP_Target;
   Trail_Stop = 0;
//----
   return(0);
  }
//== end function

//========== FUNCTION Start
int start()
  {
//----
   getOpenOrders();
   getSpread();
   //Comment(sl);
//----
   return(0);
  }
//== end function

//========== FUNCTION getPipValue
double getPipValue(double ord,int dir)
{
   double val;
   RefreshRates();
   if(dir == 1) val=(NormalizeDouble(ord,Digits) - 
NormalizeDouble(Ask,Digits));
   else val=(NormalizeDouble(Bid,Digits) - NormalizeDouble(ord,Digits));
   val = val/Point;
   return(val);
}
//== end function

int getSpread()
{
   int spread=MarketInfo(Symbol(),MODE_SPREAD);
   return(spread);
}


int getMaxLoss()
{
   int calcMaxLoss;
   calcMaxLoss = Max_Loss;
   return(calcMaxLoss);
}
//========== FUNCTION getOpenOrders
void getOpenOrders()
{
   int nsl, nsd;
   string mngMagic;
   int totalorders = OrdersTotal();
   for(int j=0; j<totalorders;j++)
     {
         OrderSelect(j, SELECT_BY_POS, MODE_TRADES);
         if((OrderType() == OP_BUY || OrderType() == OP_SELL) && 
OrderSymbol() == Symbol()&&(Magic_Number==0 || Magic_Number == 
OrderMagicNumber()))
         {

            double val=getPipValue(OrderOpenPrice(),OrderType());
            //int val = OrderProfit()/(OrderLots()*10);
            if(Use_Max_Loss && Max_Loss > 0) killTrade(val,OrderTicket());
            if(Move_Stops) checkStops(val,OrderTicket());
            if(First_TP_Target !=0) takeProfit(val,OrderTicket());

         }

         if(Magic_Number == 0)
         mngMagic = "All "+Symbol()+" trades.";
         else
         mngMagic = "Trades with magic number = "+Magic_Number;


         if(sl==0)
            {
               nsl = First_Stop_Target;
               nsd = First_Stop;
            }
         else if(sl==First_Stop)
            {
               nsl = Second_Stop_Target;
               nsd = Second_Stop;
            } 
         else if(sl==Second_Stop)
            {
               nsl = Third_Stop_Target;
               nsd = Third_Stop;
            }
         else if(Trail_Stop_By != 0 && val - Trail_Stop_By > sl)
            {
               nsl = val + 1;
               nsd = val + 1 - Trail_Stop_By; 
               trailon = " ON";  
            }
         //RefreshRates();
         Comment(version, 
         "\nPreSet ..... ", PreSet_Name,
         "\nClose Lots:  ", Close_Lots,
         "\nTP:   ", First_TP_Target, "  ", TP_Target_Increment,
         "\nSL1:  ", First_Stop_Target, "  ", First_Stop,
         "\nSL2:  ", Second_Stop_Target, "  ", Second_Stop,
         "\nSL3:  ", Third_Stop_Target,  "  ", Third_Stop,
         "\nTSL:  ", Trail_Stop_By,
         "\nMax Loss:  ", Max_Loss,
         "\n----------------------------",
         "\nOrder Open: ",OrderOpenPrice(),
         "\nPip Count: ", val,
         "\nNext TP:   ", nextTP,
         "\nCurr  SL:   ", sl,
         "\nTrail  SL:  ", trailon,
         "\nTSL Trigger:  ", Trail_Trigger,
         "\nManaging:  ", mngMagic);


     }
}

//========== FUNCTION takeProfit
void takeProfit(int pips, int ticket)
{
   if(OrderSelect(ticket, SELECT_BY_TICKET)==true)
   {

//      if(pips >= nextTP && pips < (nextTP + TP_Target_Increment))
      if(pips >= nextTP)
      {
         if(OrderType()==1)
         {
            if(OrderClose(ticket, Close_Lots, Ask, 3, YellowGreen))
            nextTP+=TP_Target_Increment;
            else
            Print("Error closing order : ",GetLastError());
         }
         else
         {
            if(OrderClose(ticket, Close_Lots, Bid, 3, YellowGreen))
            nextTP+=TP_Target_Increment;
            else
            Print("Error closing order : ",GetLastError());
         }

      }

   }
}

//== end function



//========== FUNCTION moveStops
void checkStops(int pips,int ticket)
{
   if(sl==0 && First_Stop != 0)
   {
      if(pips >= First_Stop_Target)
      {
         moveStops(ticket, First_Stop);
      }
   }
   else if(sl==First_Stop && First_Stop != 0)
   {
      if(pips >= Second_Stop_Target)
      {
         moveStops(ticket,Second_Stop);
      }
   }
   else if(sl==Second_Stop && Second_Stop != 0)
   {
      if(pips >= Third_Stop_Target)
      {
         moveStops(ticket,Third_Stop);
      }
   }
   else if(sl >= Trail_Trigger && Trail_Stop_By != 0)
   {
      if(pips - Trail_Stop_By > sl)
      {
         Trail_Stop = pips - Trail_Stop_By;
         moveStops(ticket,Trail_Stop);
         trailon = " ON";  
      }
   }      
}

//== end function

//========== FUNCTION moveStops
void moveStops(int ticket,int stopDiff)
{
   if(OrderSelect(ticket, SELECT_BY_TICKET)==true)
   {
      Print("moveStops called ",ticket, " ",stopDiff);
      if(OrderType()==1)
      {
      OrderModify(ticket,OrderOpenPrice(),OrderOpenPrice()-stopDiff*Point, 
OrderTakeProfit(),0,Plum);
      sl=stopDiff;
      }
      else
      {
      OrderModify(ticket,OrderOpenPrice(),OrderOpenPrice()+stopDiff*Point, 
OrderTakeProfit(),0,Plum);
      sl=stopDiff;
      }
   }
}

//== end function

//========== FUNCTION killTrades
void killTrade(int pips, int ticket)
{
   if(OrderSelect(ticket, SELECT_BY_TICKET)==true)
   {
      if(pips <= -1*getMaxLoss())
      {
         if(OrderType()==1) OrderClose(ticket,OrderLots(),Ask,3,Red);
         else OrderClose(ticket,OrderLots(),Bid,3,Red);
      }
   }
}
//== end function